#include "nvector.h"   /* definitions of type N_Vector, macro N_VDATA     */
#include "llnltyps.h"  /* definitions of real, integer, TRUE, FALSE       */
#include "mpi.h"       /* MPI data types and prototypes                   */

/* Problem Constants */

#define NVARS        2             /* number of species         */

#define C1_SCALE     1.0e6         /* coefficients in initial profiles    */
#define C2_SCALE     1.0e12

#define T0           0.0           /* initial time */
#define NOUT         24            /* number of output times */
#define TWOHR        3600.0        /* number of seconds in one time interval */
#define HALFDAY      4.32e4        /* number of seconds in a half day */
#define PI       3.1415926535898   /* pi */ 

#define XMIN          0.0          /* grid boundaries in x  */
#define XMAX         20.0           
#define YMIN         30.0          /* grid boundaries in y  */
#define YMAX         50.0

#define NPEX         2              /* no. PEs in x direction of PE array */
#define NPEY         2              /* no. PEs in y direction of PE array */
                                    /* Total no. PEs = NPEX*NPEY */
#define MXSUB        5              /* no. x points per subgrid */
#define MYSUB        5              /* no. y points per subgrid */

#define MX           (NPEX*MXSUB)   /* MX = number of x mesh points */
#define MY           (NPEY*MYSUB)   /* MY = number of y mesh points */
                                    /* Spatial mesh is MX by MY */

#define NY           NVARS*MX*MY    /* number of equations          */
#define NP           8              /* number of parameters         */
/* #define NS           8  */       /* number of sensitivities      */

#define ZERO         RCONST(0.0)    /* real 0.0 */
#define ONE          RCONST(1.0)    /* real 1.0 */
#define TWO          RCONST(2.0)    /* real 2.0 */

/* Type : UserData 
   contains problem constants, extended dependent variable array,
   grid constants, processor indices, MPI communicator */

typedef struct {
  real om, dx, dy, q4;
  real uext[NVARS*(MXSUB+2)*(MYSUB+2)];
  integer my_pe, isubx, isuby, nvmxsub, nvmxsub2;
  real *p;
  real *pbar;
  MPI_Comm comm;
} *UserData;


/* Prototype of function called by the CVODE solver */

void f_old(integer N, nosens_real t, N_Vector u, N_Vector udot, void *f_data);

void flocal(integer N, nosens_real t, real uarray[], real duarray[], 
                    void *f_data);

void ucomm(integer N, nosens_real t, N_Vector u, void *f_data);


